<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


if ($is_logged_in_admin===true) //redirect to the dashboard if logged in
    {
    header("Location: index.php");
    exit();
    }


$page_title="Administrator Login";
$page_message="Login to start your session.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (isset($submit_ok))
    {
    if (filter_var($admin_email, FILTER_VALIDATE_EMAIL) && !empty($admin_password) && !empty($rows_array=fetchRow("SELECT * FROM apl_admins WHERE admin_email=?", array($admin_email), array("s"))))
        {
        foreach ($rows_array as $row)
            {
            extract($row, EXTR_PREFIX_ALL, "logged"); //add prefix 'logged'

            if ($error_detected!=1)
                {
                if (password_verify($admin_password, $logged_admin_password)) //everything ok
                    {
                    $action_success=1;

                    if (!empty($remember_me) && $remember_me==1) //store cookie for 1 month
                        {
                        $cookie_expiration_time=strtotime("+1 month");
                        }
                    else //cookie will expire after browser is closed
                        {
                        $cookie_expiration_time=0;
                        }

                    setcookie(COOKIE_PREFIX."_admin_id", $logged_admin_id, $cookie_expiration_time, null, null, null, true); //id
                    setcookie(COOKIE_PREFIX."_admin_email", password_hash($logged_admin_email, PASSWORD_DEFAULT), $cookie_expiration_time, null, null, null, true); //email (encrypted with password_hash)
                    setcookie(COOKIE_PREFIX."_admin_key", password_hash($logged_admin_id.$logged_admin_email.$logged_admin_password, PASSWORD_DEFAULT), $cookie_expiration_time, null, null, null, true); //key (id, email and password hash) (encrypted with password_hash)
                    setcookie(COOKIE_PREFIX."_admin_data_authenticity", password_hash($ip_address.$user_agent, PASSWORD_DEFAULT), $cookie_expiration_time, null, null, null, true); //user agent and IP (encrypted with password_hash)

                    if (!empty($logged_admin_reset)) //reset admin_reset value (in case lost password email was sent previously, but admin didn't change his password)
                        {
                        $logged_admin_reset=null;

                        updateRow("UPDATE apl_admins SET admin_reset=? WHERE admin_id=?", array($logged_admin_reset, $logged_admin_id), array("s", "i"));
                        }
                    }
                else
                    {
                    $error_detected=1;
                    $error_details.="Invalid email address or password.<br>";
                    }
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid email address or password.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="Admin $admin_email logged in from IP address $ip_address.";
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="Admin $admin_email login from IP address $ip_address failed because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }

    if (empty($logged_admin_id) || !filter_var($logged_admin_id, FILTER_VALIDATE_INT))
        {
        $logged_admin_id=null;
        }

    createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);

    if ($action_success==1)
        {
        if (!empty($_COOKIE[COOKIE_PREFIX."_requested_page"])) //cookie with requested page set, unset cookie and redirect to this page
            {
            $requested_page_cookie=$_COOKIE[COOKIE_PREFIX."_requested_page"];
            setcookie($_COOKIE[COOKIE_PREFIX."_requested_page"], false, 1);
            header("Location: $requested_page_cookie");
            exit();
            }
        else //redirect to the dashboard
            {
            header("Location: index.php");
            exit();
            }
        }
    }


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
