<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Database Cleanup Settings";
$page_message="Configure database cleanup settings. Automatic removal of old data can slightly reduce database usage and improve overall performance.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (isset($submit_ok))
    {
    if (validateNumberOrRange($database_cleanup_enabled, 0, 1) && validateNumberOrRange($database_cleanup_callbacks, 0, 365) && validateNumberOrRange($database_cleanup_reports_main, 0, 365) && validateNumberOrRange($database_cleanup_reports_system, 0, 365) && validateNumberOrRange($database_cleanup_licenses, 0, 365))
        {
        if ($error_detected!=1)
            {
            $updated_records=updateRow("UPDATE apl_settings SET DATABASE_CLEANUP_ENABLED=?, DATABASE_CLEANUP_CALLBACKS=?, DATABASE_CLEANUP_REPORTS_MAIN=?, DATABASE_CLEANUP_REPORTS_SYSTEM=?, DATABASE_CLEANUP_LICENSES=?", array($database_cleanup_enabled, $database_cleanup_callbacks, $database_cleanup_reports_main, $database_cleanup_reports_system, $database_cleanup_licenses), array("i", "i", "i", "i", "i"));
            if ($updated_records<1)
                {
                $error_detected=1;
                $error_details.="Invalid settings or duplicated record (no new data).<br>";
                }
            else
                {
                $action_success=1;
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid time range.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="Database cleanup settings updated.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="The database could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }


    //get all settings again
    $rows_array=fetchRow("SELECT * FROM apl_settings");
    foreach ($rows_array as $row)
        {
        extract($row);
        }
    }


$database_cleanup_callbacks_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30, 60, 90, 180, 365), "Days", "Disabled", $DATABASE_CLEANUP_CALLBACKS);
$database_cleanup_reports_main_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30, 60, 90, 180, 365), "Days", "Disabled", $DATABASE_CLEANUP_REPORTS_MAIN);
$database_cleanup_reports_system_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30, 60, 90, 180, 365), "Days", "Disabled", $DATABASE_CLEANUP_REPORTS_SYSTEM);
$database_cleanup_licenses_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30, 60, 90, 180, 365), "Days ago", "Disabled", $DATABASE_CLEANUP_LICENSES);


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
