<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Software Updates";
$page_message="Check for new version and automatically update $PRODUCT_NAME to get new features, bug fixes, and performance improvements.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (!isset($submit_ok)) //check for latest version only if upgrade wasn't submitted
    {
    $license_notifications_array=aplVerifySupport($GLOBALS["mysqli"]); //check support status
    if ($license_notifications_array['notification_case']=="notification_support_expired") //support expired
        {
        $support_expired=1;

        $support_expired_title="$PRODUCT_NAME Support Expired";
        $support_expired_message="Your support is expired. You can still download updates, but if something goes wrong during upgrade, this installation might stop working.<br><br>In order to avoid any downtime, <a href='https://codecanyon.net/downloads?ref=phpmillion' target='_blank'>renew your support</a> first, or proceed at your own risk.<br><br><b>Attention</b>: if you renewed support, this message will go away within next 24 hours.";
        $support_expired_message_class="alert alert-danger";
        }

    $version_check_content=customPost(APL_ROOT_URL."/apl_callbacks/version_check.php", $ROOT_URL, "product_id=".rawurlencode(APL_PRODUCT_ID)."&product_version=".rawurlencode($PRODUCT_VERSION)."&connection_hash=".rawurlencode(hash("sha256", "version_check")));
    if (!empty($version_check_content))
        {
        $latest_product_version=aplParseXmlTags($version_check_content, "version_check");
        if (!empty($latest_product_version)) //latest version parsed successfully
            {
            if ($latest_product_version!=$PRODUCT_VERSION) //new version available
                {
                $upgrade_available=1;

                $page_message="$PRODUCT_NAME version $latest_product_version available! Upgrade now for new features, bug fixes, and performance improvements.<br><br>If you like continuous $PRODUCT_NAME updates, can you <a href='$PRODUCT_REVIEW_PAGE' target='_blank'><b>leave your feedback</b></a>, please? Here's the deal: your positive rating might help us to get an extra sale or two, so we have enough resources to provide you with future updates.";
                $page_message_class="alert alert-warning";
                }
            else //no newer version available
                {
                $page_message="You have the latest $PRODUCT_NAME version installed! Once a new version is available, automatic upgrade option will appear below.";
                $page_message_class="alert alert-success";
                }
            }
        else //parsing failed
            {
            $page_message="Impossible to parse the latest $PRODUCT_NAME version. Try again later...";
            $page_message_class="alert alert-warning";
            }
        }
    else //no connection
        {
        $page_message="Impossible to connect to $PRODUCT_NAME server. Try again later...";
        $page_message_class="alert alert-danger";
        }
    }


if (isset($submit_ok))
    {
    $post_info="product_id=".rawurlencode(APL_PRODUCT_ID)."&product_version=".rawurlencode($PRODUCT_VERSION)."&client_email=".rawurlencode($CLIENT_EMAIL)."&license_code=".rawurlencode($LICENSE_CODE)."&root_url=".rawurlencode($ROOT_URL)."&installation_hash=".rawurlencode($INSTALLATION_HASH)."&license_signature=".rawurlencode(aplGenerateScriptSignature($ROOT_URL, $CLIENT_EMAIL, $LICENSE_CODE));
    $upgrade_archive_content=customPost(APL_ROOT_URL."/apl_callbacks/software_download.php", $ROOT_URL, $post_info); //will return zip file on success or error message on failure
    if (!empty($upgrade_archive_content))
        {
        if (substr($upgrade_archive_content, 0, 2)!="PK") //something else instead of zip archive returned
            {
            if (stristr($upgrade_archive_content, "<notification_") && stristr($upgrade_archive_content, "</notification_")) //a specific error notification returned
                {
                $notifications_array=aplParseServerNotifications($upgrade_archive_content); //parse <notification_case> along with message

                $error_detected=1;
                $error_details.=$notifications_array['notification_text'].".<br>";
                }
            else
                {
                $error_detected=1;
                $error_details.="No valid content from $PRODUCT_NAME server received.<br>";
                }
            }
        else //everything ok, zip archive returned
            {
            $zip_file_name=str_ireplace(" ", "-", $PRODUCT_NAME).".zip"; //zip archive name should look as Product-Name.zip
            $upgrade_archive_local_destination=SCRIPT_ROOT_DIRECTORY."/$zip_file_name"; //download archive right to the script's root directory

            $zip_file=@fopen($upgrade_archive_local_destination, "w+");
            $fwrite=@fwrite($zip_file, $upgrade_archive_content);
            if (!filter_var($fwrite, FILTER_VALIDATE_INT)) //saving zip archive failed
                {
                $error_detected=1;
                $error_details.="Impossible to save upgrade archive at ".SCRIPT_ROOT_DIRECTORY.".<br>";
                }
            else //zip archive dowloaded and saved, extract it
                {
                $zip_file=new ZipArchive;
                if ($zip_file->open(SCRIPT_ROOT_DIRECTORY."/$zip_file_name")!=true) //zip archive can't be opened
                    {
                    $error_detected=1;
                    $error_details.="Impossible to open and/or extract upgrade archive at ".SCRIPT_ROOT_DIRECTORY.".<br>";
                    }
                else //everything ok, extract zip archive
                    {
                    $zip_file->extractTo(SCRIPT_ROOT_DIRECTORY);
                    $zip_file->close();
                    $local_upgrade_content=customGet("$ROOT_URL/apl_upgrade.php", "", ""); //launch upgrade file, it will execute MySQL upgrade query and/or perform other actions in background
                    if (!empty($local_upgrade_content))
                        {
                        $upgraded_product_version=aplParseXmlTags($local_upgrade_content, "upgrade");
                        if (!empty($upgraded_product_version)) //upgrade succeeded
                            {
                            deleteFileDirectory(SCRIPT_ROOT_DIRECTORY, array("apl_install", "apl_config_sample.php", "apl_upgrade.php", $zip_file_name)); //delete sample config file, upgrade file, upgrade archive
                            $action_success=1;
                            }
                        else //upgrade failed because either the latest version is installed (upgrade already performed), either another error occurred
                            {
                            $installed_product_version=aplParseXmlTags($local_upgrade_content, "installed");
                            if (!empty($installed_product_version)) //the latest version is installed or upgrade already performed
                                {
                                $error_detected=1;
                                $error_details.="Version $installed_product_version is already installed.<br>";
                                }
                            else
                                {
                                $error_detected=1;
                                $error_details.="Upgrade script at $ROOT_URL/apl_upgrade.php failed.<br>";
                                }
                            }
                        }
                    else //no connection
                        {
                        $error_detected=1;
                        $error_details.="Impossible to connect to upgrade script at $ROOT_URL/apl_upgrade.php.<br>";
                        }
                    }
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Impossible to connect to $PRODUCT_NAME server to download the latest version.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="$PRODUCT_NAME updated from version $PRODUCT_VERSION to $upgraded_product_version.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="$PRODUCT_NAME could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }
    }


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
