<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");


if (isset($_SERVER['REMOTE_ADDR'])) {$ip_address=$_SERVER['REMOTE_ADDR'];}
if (isset($_SERVER['HTTP_REFERER'])) {$refer=$_SERVER['HTTP_REFERER'];}
if (isset($_SERVER['REQUEST_URI'])) {$requested_page=$_SERVER['REQUEST_URI'];}
if (isset($_SERVER['SCRIPT_FILENAME'])) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);}
if (isset($_SERVER['HTTP_USER_AGENT'])) {$user_agent=$_SERVER['HTTP_USER_AGENT'];}


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


$api_action_success=0;
$api_error_detected=0;
$api_error_details=null;
$logged_admin_id=null; //used for compatibility with createReport function in the same file in /apl_admin directory. since admin is not logged in when API is called, $logged_admin_id must be null


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (!empty($api_key_secret) && !empty($api_function) && $api_post_key==hash("sha256", $ROOT_URL) && $submit_ok=="Submit" && $refer=="$ROOT_URL/apl_api/api.php" && in_array($user_agent, $SUPPORTED_BROWSERS_ARRAY)) //prevent someone from posting to this file directly
    {
    if ($API_STATUS==1 && in_array($api_function, $SUPPORTED_API_FUNCTIONS_ARRAY))
        {
        $api_key_rows_array=fetchRow("SELECT * FROM apl_api_keys WHERE api_key_secret=? AND api_key_status=?", array($api_key_secret, 1), array("s", "i"));
        if (empty($api_key_rows_array))
            {
            $api_error_detected=1;
            $api_error_details.="Invalid or inactive API key.<br>";
            }
        else
            {
            foreach ($api_key_rows_array as $api_key_row)
                {
                extract($api_key_row);
                }

            if (!empty($api_key_ip))
                {
                $api_key_ip_array=explode(",", $api_key_ip);
                if (!in_array($ip_address, $api_key_ip_array))
                    {
                    $api_error_detected=1;
                    $api_error_details.="Invalid IP address.<br>";
                    }
                }

            $api_permissions_name="api_key_".$api_function; //since each permission in database starts with api_key_ prefix, add this prefix to name of function submitted by user for quick permissions check
            if ($$api_permissions_name!=1)
                {
                $api_error_detected=1;
                $api_error_details.="Invalid API key permissions.<br>";
                }

            if ($api_error_detected!=1 && $$api_permissions_name==1)
                {
                $api_action_success=1;
                }
            }
        }
    else
        {
        $api_error_detected=1;
        $api_error_details.="API not enabled or invalid API function.<br>";
        }

    if ($api_action_success==1) //everything OK
        {
        $optional_api_parameters_array=array("license_order_number", "license_ip", "license_domain", "license_limit", "license_expire_date", "license_updates_date", "license_support_date", "license_comments"); //optional API parameters for this page
        foreach ($optional_api_parameters_array as $optional_api_parameter) //in case some required parameter was not submitted, set its value to null to prevent "undefined variable" errors
            {
            if (!isset($$optional_api_parameter))
                {
                $$optional_api_parameter=null;
                }
            }

        if (isset($submit_ok)) //code between {} tags is identical in files with the same name in /apl_admin and /apl_api directories
            {
            if (filter_var($product_id, FILTER_VALIDATE_INT) && validateNumberOrRange($license_require_domain, 0, 1) && validateNumberOrRange($license_status, 0, 2))
                {
                if (empty($client_id) || !filter_var($client_id, FILTER_VALIDATE_INT)) //in case no client_id was submitted, its value must be stored as NULL in database
                    {
                    $client_id=null;
                    }

                if (empty($license_code)) //in case no license_code was submitted, its value must be stored as NULL in database
                    {
                    $license_code=null;
                    }

                if (!filter_var($client_id, FILTER_VALIDATE_INT) && empty($license_code))
                    {
                    $error_detected=1;
                    $error_details.="Invalid client or license code.<br>";
                    }

                if (filter_var($client_id, FILTER_VALIDATE_INT) && !empty($license_code))
                    {
                    $error_detected=1;
                    $error_details.="Invalid license type (license can only be either personal (client-based) or anonymous (code-based)).<br>";
                    }

                if (!empty($license_ip))
                    {
                    $license_ip_array=explode(",", $license_ip);
                    foreach ($license_ip_array as $license_ip_array_key=>$license_ip_array_value)
                        {
                        if (!filter_var($license_ip_array_value, FILTER_VALIDATE_IP))
                            {
                            $error_detected=1;
                            $error_details.="Invalid IP address.<br>";
                            break;
                            }
                        }
                    }

                if (!empty($license_domain))
                    {
                    $license_domain_array=explode(",", $license_domain);
                    foreach ($license_domain_array as $license_domain_array_key=>$license_domain_array_value)
                        {
                        if (!validateRawDomain(getRawDomain($license_domain_array_value)) || filter_var($license_domain_array_value, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED) || !ctype_alnum(substr($license_domain_array_value, -1))) //invalid TLD, scheme included, or last symbol is not alphanumeric (most likely ends with / or another non-alphanumeric character)
                            {
                            $error_detected=1;
                            $error_details.="Invalid domain.<br>";
                            break;
                            }
                        }
                    }

                if (!empty($license_limit) && !filter_var($license_limit, FILTER_VALIDATE_INT))
                    {
                    $error_detected=1;
                    $error_details.="Invalid installations limit.<br>";
                    }

                if (!empty($license_expire_date) && !verifyDate($license_expire_date, "Y-m-d"))
                    {
                    $error_detected=1;
                    $error_details.="Invalid license expiration date.<br>";
                    }

                if (!empty($license_updates_date) && !verifyDate($license_updates_date, "Y-m-d"))
                    {
                    $error_detected=1;
                    $error_details.="Invalid updates expiration date.<br>";
                    }

                if (!empty($license_support_date) && !verifyDate($license_support_date, "Y-m-d"))
                    {
                    $error_detected=1;
                    $error_details.="Invalid support expiration date.<br>";
                    }

                if ($error_detected!=1)
                    {
                    $license_date=date("Y-m-d");

                    if (empty($license_envato) || !filter_var($license_envato, FILTER_VALIDATE_INT))
                        {
                        $license_envato=0;
                        }

                    if ($license_status==1)
                        {
                        $license_cancel_date="0000-00-00";
                        }
                    else
                        {
                        if (empty($license_cancel_date) || !verifyDate($license_cancel_date, "Y-m-d")) //set cancel date to now only if license is inactive and no previous cancel date set
                            {
                            $license_cancel_date=date("Y-m-d");
                            }
                        }

                    $added_records=insertRow("INSERT IGNORE INTO apl_licenses (client_id, license_code, product_id, license_order_number, license_ip, license_domain, license_require_domain, license_limit, license_date, license_cancel_date, license_expire_date, license_updates_date, license_support_date, license_comments, license_envato, license_status) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", array($client_id, $license_code, $product_id, $license_order_number, $license_ip, $license_domain, $license_require_domain, $license_limit, $license_date, $license_cancel_date, $license_expire_date, $license_updates_date, $license_support_date, $license_comments, $license_envato, $license_status), array("i", "s", "i", "s", "s", "s", "i", "i", "s", "s", "s", "s", "s", "s", "i", "i"));
                    if ($added_records<1)
                        {
                        $error_detected=1;
                        $error_details.="Invalid record details or duplicated record (no new data).<br>";
                        }
                    else
                        {
                        $action_success=1;

                        $license_id=mysqli_insert_id($GLOBALS["mysqli"]);
                        if (filter_var($license_id, FILTER_VALIDATE_INT))
                            {
                            $rows_array=fetchRow("SELECT * FROM apl_licenses LEFT JOIN apl_products ON apl_licenses.product_id=apl_products.product_id LEFT JOIN apl_clients ON apl_clients.client_id=apl_licenses.client_id WHERE apl_licenses.license_id=?", array($license_id), array("i")); //fetch product and client details to be used in reports
                            foreach ($rows_array as $row)
                                {
                                extract($row);
                                }

                            $client_formatted=formatClient($client_id, $client_fname, $client_lname, $client_email, $license_code);
                            }
                        }
                    }
                }
            else
                {
                $error_detected=1;
                $error_details.="Invalid product or license status.<br>";
                }

            if ($action_success==1) //everything OK
                {
                $page_message="$product_title license for $client_formatted added to the database.";
                createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
                $page_message_class="alert alert-success";
                }
            else //display error message
                {
                $page_message="The database could not be updated because of this error: <br><br>$error_details";
                $page_message_class="alert alert-danger";
                }
            }
        }
    else //display error message
        {
        $page_message="The action could not be completed because of this error:<br><br>$api_error_details";
        }


    echo $page_message;
    }
