<?php
//core functions used in all phpmillion apps


//encrypt text with custom key
function customEncrypt($string, $key)
    {
    $encrypted_string=null;

    if (!empty($string) && !empty($key))
        {
        $key=base64_decode($key); //remove the base64 encoding from key (if any)

        $iv=openssl_random_pseudo_bytes(openssl_cipher_iv_length("aes-256-cbc")); //generate an initialization vector

        $encrypted_string=openssl_encrypt($string, "aes-256-cbc", $key, 0, $iv); //encrypt the string using AES 256 encryption in CBC mode using encryption key and initialization vector
        $encrypted_string=base64_encode($encrypted_string."::".$iv); //the $iv is just as important as the key for decrypting, so save it with encrypted string using a unique separator "::"
        }

    return $encrypted_string;
    }


//decrypt text with custom key
function customDecrypt($string, $key)
    {
    $decrypted_string=null;

    if (!empty($string) && !empty($key))
        {
        $key=base64_decode($key); //remove the base64 encoding from key (if any)

        list($encrypted_string, $iv)=explode("::", base64_decode($string), 2); //to decrypt, split the encrypted string from our $iv - our unique separator used was "::"

        $decrypted_string=openssl_decrypt($encrypted_string, "aes-256-cbc", $key, 0, $iv);
        }

    return $decrypted_string;
    }


//pad encryption salt (key) with \0 because PHP >=5.6 only accepts 16/24/32 symbols (used to maintain compatibility with old keys)
function padEncryptionSalt($salt)
    {
    $allowed_lengths_array=array(16, 24, 32); //set allowed length
    $allowed_length_max=max($allowed_lengths_array);

    if (strlen($salt)>$allowed_length_max) //salt too long, cut it
        {
        $salt=substr($salt, 0, $allowed_length_max);
        }

    if (strlen($salt)<$allowed_length_max) //salt too short, pad with zeros
        {
        foreach ($allowed_lengths_array as $length) //loop through length and pad salt
            {
            while (strlen($salt)<$length)
                {
                $salt=$salt."\0";
                }

            if (strlen($salt)==$length) //nearest length reached
                {
                break;
                }
            }
        }

    return $salt;
    }


//generate random salt
function generateSalt($salt_length)
    {
    $final_salt=null;
    $symbols_list="1234567890qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM";
    $symbols_length=strlen($symbols_list);

    if (filter_var($salt_length, FILTER_VALIDATE_INT))
        {
        for ($i=0; $i<$salt_length; $i++)
            {
            $random_symbol=substr($symbols_list, mt_rand(0, $symbols_length-1), 1);
            $final_salt.=$random_symbol;
            }
        }

    return $final_salt;
    }


//check if variable is empty (standard PHP function returns true when variable is "0" (string) and has problems with constants prior ver 5.5)
function isEmpty($var)
    {
    if (empty($var) && strlen(trim($var))==0)
        {
        $result=true;
        }
    else
        {
        $result=false;
        }

    return $result;
    }


//validate numbers (or ranges like 1-10) and check if they match min and max values
function validateNumberOrRange($number, $min_value, $max_value=INF)
    {
    $result=false;

    if (filter_var($number, FILTER_VALIDATE_INT)===0 || !filter_var($number, FILTER_VALIDATE_INT)===false) //number provided
        {
        if ($number>=$min_value && $number<=$max_value)
            {
            $result=true;
            }
        else
            {
            $result=false;
            }
        }

    if (stristr($number, "-")) //range provided
        {
        $numbers_array=explode("-", $number);
        if (filter_var($numbers_array[0], FILTER_VALIDATE_INT)===0 || !filter_var($numbers_array[0], FILTER_VALIDATE_INT)===false && filter_var($numbers_array[1], FILTER_VALIDATE_INT)===0 || !filter_var($numbers_array[1], FILTER_VALIDATE_INT)===false)
            {
            if ($numbers_array[0]>=$min_value && $numbers_array[1]<=$max_value && $numbers_array[0]<=$numbers_array[1])
                {
                $result=true;
                }
            else
                {
                $result=false;
                }
            }
        }

    return $result;
    }


//get current page url (also remove specific strings and last slash if needed)
function getCurrentUrl($remove_last_slash=null, $string_to_remove_array=null)
    {
    $current_url=null;

    $protocol=!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS']!=="off" ? 'https' : 'http';
    if (isset($_SERVER['HTTP_HOST'])) {$host=$_SERVER['HTTP_HOST'];}
    if (isset($_SERVER['SCRIPT_NAME'])) {$script=$_SERVER['SCRIPT_NAME'];}
    if (isset($_SERVER['QUERY_STRING'])) {$params=$_SERVER['QUERY_STRING'];}

    if (!empty($protocol) && !empty($host) && !empty($script)) //return URL only when script is executed via browser (because no URL should exist when executed from command line)
        {
        $current_url=$protocol.'://'.$host.$script;

        if (!empty($params))
            {
            $current_url.='?'.$params;
            }

        if (!empty($string_to_remove_array) && is_array($string_to_remove_array)) //remove specific strings from URL
            {
            foreach ($string_to_remove_array as $key=>$value)
                {
                $current_url=str_ireplace($value, "", $current_url);
                }
            }

        if ($remove_last_slash==1) //remove / from the end of URL if it exists
            {
            while (substr($current_url, -1)=="/") //use cycle in case URL already contained multiple // at the end
                {
                $current_url=substr($current_url, 0, -1);
                }
            }
        }

    return $current_url;
    }


//get raw domain (returns (sub.)domain.com from url like http://www.(sub.)domain.com/something.php?xx=yy)
function getRawDomain($url)
    {
    $raw_domain=null;

    if (!empty($url))
        {
        $url_array=parse_url($url);
        if (empty($url_array['scheme'])) //in case no scheme was provided in url, it will be parsed incorrectly. add http:// and re-parse
            {
            $url="http://".$url;
            $url_array=parse_url($url);
            }

        if (!empty($url_array['host']))
            {
            $raw_domain=$url_array['host'];

            $raw_domain=trim(str_ireplace("www.", "", filter_var($raw_domain, FILTER_SANITIZE_URL)));
            }
        }

    return $raw_domain;
    }


//validate raw domain (only URL like (sub.)domain.com will validate)
function validateRawDomain($url)
    {
    $result=false;

    if (!empty($url))
        {
        if (preg_match('/^[a-z0-9-.]+\.[a-z\.]{2,7}$/', strtolower($url))) //check if this is valid tld
            {
            $result=true;
            }
        else
            {
            $result=false;
            }
        }

    return $result;
    }


//return root url from long url (http://www.domain.com/path/file.php?aa=xx becomes http://www.domain.com/path/), remove scheme, www. and last slash if needed
function getRootUrl($url, $remove_scheme, $remove_www, $remove_path, $remove_last_slash)
    {
    if (filter_var($url, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED))
        {
        $url_array=parse_url($url); //parse URL into arrays like $url_array['scheme'], $url_array['host'], etc

        $url=str_ireplace($url_array['scheme']."://", "", $url); //make URL without scheme, so no :// is included when searching for first or last /

        if ($remove_path==1) //remove everything after FIRST / in URL, so it becomes "real" root URL
            {
            $first_slash_position=stripos($url, "/"); //find FIRST slash - the end of root URL
            if ($first_slash_position>0) //cut URL up to FIRST slash
                {
                $url=substr($url, 0, $first_slash_position+1);
                }
            }
        else //remove everything after LAST / in URL, so it becomes "normal" root URL
            {
            $last_slash_position=strripos($url, "/"); //find LAST slash - the end of root URL
            if ($last_slash_position>0) //cut URL up to LAST slash
                {
                $url=substr($url, 0, $last_slash_position+1);
                }
            }

        if ($remove_scheme!=1) //scheme was already removed, add it again
            {
            $url=$url_array['scheme']."://".$url;
            }

        if ($remove_www==1) //remove www.
            {
            $url=str_ireplace("www.", "", $url);
            }

        if ($remove_last_slash==1) //remove / from the end of URL if it exists
            {
            while (substr($url, -1)=="/") //use cycle in case URL already contained multiple // at the end
                {
                $url=substr($url, 0, -1);
                }
            }
        }

    return trim($url);
    }


//make get requests with cookies, referrers, etc
function customGet($url, $refer=null, $header_array=null)
    {
    $user_agent="Mozilla/5.0 (Windows NT 6.3; WOW64; rv:48.0) Gecko/20100101 Firefox/48.0"; //set user agent
    $connect_timeout=10; //set connection timeout

    if (empty($refer) || !filter_var($refer, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED)) {$refer=$url;} //use original url as refer when no valid URL provided

    $ch=curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $connect_timeout);
    curl_setopt($ch, CURLOPT_TIMEOUT, $connect_timeout);
    curl_setopt($ch, CURLOPT_REFERER, $refer);
    if (!empty($header_array) && is_array($header_array)) //add custom header
        {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header_array);
        }
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
    $result=curl_exec($ch);
    curl_close($ch);

    return $result;
    }


//make post requests with cookies, referrers, etc
function customPost($url, $refer=null, $post_info=null)
    {
    $user_agent="Mozilla/5.0 (Windows NT 6.3; WOW64; rv:48.0) Gecko/20100101 Firefox/48.0"; //set user agent
    $connect_timeout=10; //set connection timeout

    if (empty($refer) || !filter_var($refer, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED)) {$refer=$url;} //use original url as refer when no valid URL provided

    $ch=curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $connect_timeout);
    curl_setopt($ch, CURLOPT_TIMEOUT, $connect_timeout);
    curl_setopt($ch, CURLOPT_REFERER, $refer);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post_info);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
    $result=curl_exec($ch);
    curl_close($ch);

    return $result;
    }


//verify date according to provided format (such as Y-m-d)
function verifyDate($date, $date_format)
    {
    $datetime=DateTime::createFromFormat($date_format, $date);
    $errors=DateTime::getLastErrors();
    if (!$datetime || !empty($errors['warning_count'])) //date was invalid
        {
        $date_check_ok=false;
        }
    else //everything OK
        {
        $date_check_ok=true;
        }

    return $date_check_ok;
    }


//calculate number of days between dates
function getDaysBetweenDates($date_from, $date_to)
    {
    $number_of_days=0;

    if (verifyDate($date_from, "Y-m-d") && verifyDate($date_to, "Y-m-d"))
        {
        $date_to=new DateTime($date_to);
        $date_from=new DateTime($date_from);
        $number_of_days=$date_from->diff($date_to)->format("%a");
        }

    return $number_of_days;
    }


//remove elements from standard or multi-dimensional arrays that don't match min/max length, preserving same keys
function cleanupArray($array, $min_length=1, $max_length=INF)
    {
    $root_array=array();

    if (!empty($array) && is_array($array) && filter_var($min_length, FILTER_VALIDATE_INT))
        {
        foreach ($array as $key=>$value)
            {
            if (is_array($value)) //$value is a sub-array
                {
                foreach ($value as $subkey=>$subvalue)
                    {
                    if (strlen(trim($subvalue))>=$min_length && strlen(trim($subvalue))<=$max_length) //check if subvalue meets min/max length
                        {
                        $root_array[$key][$subkey]=trim($subvalue);
                        }
                    }
                }
            else //$value is a string
                {
                if (strlen(trim($value))>=$min_length && strlen(trim($value))<=$max_length) //check if value meets min/max length
                    {
                    $root_array[$key]=trim($value);
                    }
                }
            }
        }

    return $root_array;
    }


//convert objects into arrays
function convertObjectToArray($arrObjData, $arrSkipIndices=array())
    {
    $arrData=array();

    if (is_object($arrObjData)) //if input is object, convert into array
        {
        $arrObjData = get_object_vars($arrObjData);
        }

    if (is_array($arrObjData))
        {
        foreach ($arrObjData as $index=>$value)
            {
            if (is_object($value) || is_array($value))
                {
                $value=convertObjectToArray($value, $arrSkipIndices); //recursive call
                }
            if (in_array($index, $arrSkipIndices))
                {
                continue;
                }
            $arrData[$index]=$value;
            }
        }

    return $arrData;
    }


//convert variables into arrays
function convertVariableToArray($var_name)
    {
    if (!is_array($var_name))
        {
        $var_name=array($var_name);
        }

    return $var_name;
    }


//sort multi-dimensional arrays by specified key. no quotes for sort_order! call function directly like sortMultiArray($array, "sort_key", SORT_DESC) and NOT like $array=sortMultiArray($array, "sort_key", "SORT_DESC")! https://stackoverflow.com/a/2699153
function sortMultiArray(&$arr, $sort_key, $sort_order)
    {
    $sort_col=array();

    foreach ($arr as $key=>$row)
        {
        $sort_col[$key]=$row[$sort_key];
        }

    array_multisort($sort_col, $sort_order, $arr);
    }


//create slug from string
function slugifyText($string)
    {
    if (!empty($string))
        {
        $string=strtolower(preg_replace("/[^A-Za-z0-9]+/", "-", $string));

        while (substr($string, -1)=="-") //remove excessive "-" from the end
            {
            $string=substr($string, 0, -1);
            }
        }

    return $string;
    }


//remove invisible HTML tags, such as style and script code, embedded objects, etc (use before strip_tags because strip_tags only removes tags themselves and leaves content between them)
function removeInvisibleHtml($content)
    {
    if (!empty($content)) //$content can be either string, either array
        {
        $content=preg_replace(
        array(
            '@<!--[^>]*?.*?-->@siu',
            '@<applet[^>]*?.*?</applet>@siu',
            '@<area[^>]*?.*?</area>@siu',
            '@<audio[^>]*?.*?</audio>@siu',
            '@<base[^>]*?.*?</base>@siu',
            '@<body[^>]*?.*?</body>@siu',
            '@<button[^>]*?.*?</button>@siu',
            '@<canvas[^>]*?.*?</canvas>@siu',
            '@<datalist[^>]*?.*?</datalist>@siu',
            '@<embed[^>]*?.*?</embed>@siu',
            '@<fieldset[^>]*?.*?</fieldset>@siu',
            '@<figure[^>]*?.*?</figure>@siu',
            '@<form[^>]*?.*?</form>@siu',
            '@<frame[^>]*?.*?</frame>@siu',
            '@<frameset[^>]*?.*?</frameset>@siu',
            '@<head[^>]*?>.*?</head>@siu',
            '@<html[^>]*?>.*?</html>@siu',
            '@<iframe[^>]*?.*?</iframe>@siu',
            '@<input[^>]*?.*?>@siu',
            '@<keygen[^>]*?.*?</keygen>@siu',
            '@<map[^>]*?.*?</map>@siu',
            '@<meta[^>]*?.*?</meta>@siu',
            '@<noembed[^>]*?.*?</noembed>@siu',
            '@<noframes[^>]*?.*?</noframes>@siu',
            '@<noscript[^>]*?.*?</noscript>@siu',
            '@<object[^>]*?.*?</object>@siu',
            '@<optgroup[^>]*?.*?</optgroup>@siu',
            '@<option[^>]*?.*?</option>@siu',
            '@<output[^>]*?.*?</output>@siu',
            '@<param[^>]*?.*?</param>@siu',
            '@<script[^>]*?.*?</script>@siu',
            '@<select[^>]*?.*?</select>@siu',
            '@<source[^>]*?.*?</source>@siu',
            '@<textarea[^>]*?.*?</textarea>@siu',
            '@<title[^>]*?.*?</title>@siu',
            '@<track[^>]*?.*?</track>@siu',
            '@<video[^>]*?.*?</video>@siu'
        ), "", $content);
        }

    return $content;
    }


//remove all tags and excess whitespace ($content can be either string, either array)
function cleanupContent($content, $allow_html_tags=0)
    {
    $content=removeInvisibleHtml($content); //remove invisible HTML tags first (use before strip_tags because strip_tags only removes tags themselves and leaves content between them)

    if ($allow_html_tags!=1) //remove all tags, unless specified otherwise
        {
        if (is_array($content))
            {
            $content=array_map("strip_tags", $content);
            }
        else
            {
            $content=strip_tags($content);
            }
        }

    $content=preg_replace("/( )+/", " ", $content); //replace excess whitespace with single whitespace, preserving new lines

    if (is_array($content)) //finally trim content
        {
        $content=array_map("trim", $content);
        }
    else
        {
        $content=trim($content);
        }

	return $content;
    }


//sanitize data submitted by user via $_GET/$_POST (designed to work with array_walk() mainly)
function sanitizeSubmittedData(&$value, $key, $custom_data_array)
    {
    $allow_html_tags=0;

    $script_filename=$custom_data_array['script_filename']; //filename of page user requested
    $html_fields=$custom_data_array['html_fields']; //array with form fields that allow HTML-like tags (where key is script filename and value is a sub-array with names of form fields that allow HTML)

    if (!empty($html_fields[$script_filename]) && in_array($key, $html_fields[$script_filename])) //requested page has no fields with HTML tags allowed, remove them
        {
        $allow_html_tags=1;
        }

    $value=cleanupContent($value, $allow_html_tags);
    }


//format and return status text and class array
function returnFormattedStatusArray($status, $active_text, $inactive_text, $suspended_text)
    {
    if ($status==0) {$item_array['status_text']=$inactive_text; $item_array['status_class']="label label-danger";}
    if ($status==1) {$item_array['status_text']=$active_text; $item_array['status_class']="label label-success";}
    if ($status==2) {$item_array['status_text']=$suspended_text; $item_array['status_class']="label label-warning";}

    $root_array=$item_array;

    return $root_array;
    }


//format and return green/red report status and text depending on report success/failure status
function returnFormattedReportStatusArray($report_failure, $success_text, $failure_text, $warning_text)
    {
    if ($report_failure==0) {$item_array['status_text']=$success_text; $item_array['status_class']="label label-success";}
    if ($report_failure==1) {$item_array['status_text']=$failure_text; $item_array['status_class']="label label-danger";}
    if ($report_failure==2) {$item_array['status_text']=$warning_text; $item_array['status_class']="label label-warning";}

    $root_array=$item_array;

    return $root_array;
    }


//format system report user
function formatSystemReportUser($admin_email)
    {
    if (filter_var($admin_email, FILTER_VALIDATE_EMAIL))
        {
        $user_formatted=$admin_email;
        }
    else
        {
        $user_formatted="System";
        }

    return $user_formatted;
    }


//truncate text by number of symbols or words (text will be limited up to $max_length symbols/words)
function truncateText($content, $max_length, $truncate_type, $finish_sentence, $add_final_dot)
    {
    $text_truncated=null;

    if (filter_var($max_length, FILTER_VALIDATE_INT)) //in case no limit is specified, don't truncate text
        {
        if ($truncate_type=="symbols") //truncate text by symbols
            {
            $current_length=strlen($content);
            if ($current_length>$max_length) //content length exceeded, truncate
                {
                $content=preg_replace("/\s+?(\S+)?$/", "", substr($content, 0, $max_length));
                $text_truncated=1; //will be used when ... needs to be added at the end (so ... is not added to text which was not truncated)
                }
            }

        if ($truncate_type=="words") //truncate text by words
            {
            $content=preg_replace("/\s\s+/", " ", $content); //replace new lines and excess whitespace with single whitespace
            $words_array=explode(" ", $content); //make an array of individual words

            if (count($words_array)>$max_length) //content length exceeded, truncate
                {
                $words_array=array_slice($words_array, 0, $max_length);

                $content=implode(" ", $words_array);
                $text_truncated=1; //will be used when ... needs to be added at the end (so ... is not added to text which was not truncated)
                }
            }

        $content=trim($content);
        }

    $end_symbols_array=array(".", "?", "!"); //an array with symbols that can be used at the end of text
    $last_symbol=substr($content, -1); //get the last symbol of current content

    if ($finish_sentence==1) //truncate content at the end of sentence, so it doesn't look like "this is some..."
        {
        while (!in_array($last_symbol, $end_symbols_array)) //truncate text just when one of end symbols is detected
            {
            $content=substr($content, 0, -1);
            $last_symbol=substr($content, -1);
            }
        }

    if ($add_final_dot==1 && !in_array($last_symbol, $end_symbols_array)) //add . to the end if needed
        {
        $content.=".";
        }

    if ($add_final_dot==2 && $text_truncated==1 && !in_array($last_symbol, $end_symbols_array)) //add ... to the end if needed
        {
        $content.="...";
        }

    return $content;
    }


//format and return nice numbers dropdown array
function returnNumbersDropdownArray($numbers_array, $title, $disabled_title, $current_value)
    {
    $root_array=array();

    if (is_array($numbers_array) && !empty($title) && !empty($disabled_title))
        {
        foreach ($numbers_array as $value)
            {
            $item_array['value']=$value;
            $item_array['title']=$value; //use $value as option title by default

            if ($value==0 && !empty($disabled_title)) //format disabled title when value is 0
                {
                $item_array['title']=$disabled_title;
                }

            if ($value>0 && !empty($title)) //format title by adding a specific word to it
                {
                $item_array['title'].=" $title";

                if ($value==1 && substr($item_array['title'], -1)=="s") //remove last "s" if needed, so "1 records" becomes "1 record"
                    {
                    $item_array['title']=substr($item_array['title'], 0, -1);
                    }
                }

            if ($value==$current_value) //this value should be selected
                {
                $item_array['selected']=" selected";
                }
            else
                {
                $item_array['selected']=null;
                }

            $root_array[]=$item_array;
            }
        }

    return $root_array;
    }


//convert values to references (used by call_user_func_array function)
function valuesToReferences(&$values_array)
    {
    $references=array();

    foreach ($values_array as $key=>$value)
        {
        $references[$key]=&$values_array[$key];
        }

    return $references;
    }


//insert row by auto binding parameters to provided mysql query and return number of inserted rows
function insertRow($formatted_query, $column_values_array, $column_parameters_array)
    {
    $affected_rows=0;

    if (!empty($formatted_query) && !empty($column_values_array) && is_array($column_values_array) && !empty($column_parameters_array) && is_array($column_parameters_array) && count($column_values_array)==count($column_parameters_array))
        {
        $stmt=mysqli_prepare($GLOBALS["mysqli"], $formatted_query);
        if ($stmt)
            {
            call_user_func_array("mysqli_stmt_bind_param", array_merge(array($stmt, implode($column_parameters_array)), valuesToReferences($column_values_array)));
            $exec=mysqli_stmt_execute($stmt);
            $affected_rows=mysqli_stmt_affected_rows($stmt);
            mysqli_stmt_close($stmt);
            }
        }

    return $affected_rows;
    }


//update row by auto binding parameters to provided mysql query and return number of updated rows
function updateRow($formatted_query, $column_values_array, $column_parameters_array)
    {
    $affected_rows=0;

    if (!empty($formatted_query) && !empty($column_values_array) && is_array($column_values_array) && !empty($column_parameters_array) && is_array($column_parameters_array) && count($column_values_array)==count($column_parameters_array))
        {
        $stmt=mysqli_prepare($GLOBALS["mysqli"], $formatted_query);
        if ($stmt)
            {
            call_user_func_array("mysqli_stmt_bind_param", array_merge(array($stmt, implode($column_parameters_array)), valuesToReferences($column_values_array)));
            $exec=mysqli_stmt_execute($stmt);
            $affected_rows=mysqli_stmt_affected_rows($stmt);
            mysqli_stmt_close($stmt);
            }
        }

    return $affected_rows;
    }


//delete row by auto binding parameters to provided mysql query and return number of deleted rows
function deleteRow($formatted_query, $column_values_array=null, $column_parameters_array=null)
    {
    $affected_rows=0;

    if (!empty($formatted_query))
        {
        $stmt=mysqli_prepare($GLOBALS["mysqli"], $formatted_query);
        if ($stmt)
            {
            if (!empty($column_values_array) && is_array($column_values_array) && !empty($column_parameters_array) && is_array($column_parameters_array) && count($column_values_array)==count($column_parameters_array)) //delete specified row by binding parameters
                {
                call_user_func_array("mysqli_stmt_bind_param", array_merge(array($stmt, implode($column_parameters_array)), valuesToReferences($column_values_array)));
                }
            $exec=mysqli_stmt_execute($stmt);
            $affected_rows=mysqli_stmt_affected_rows($stmt);
            mysqli_stmt_close($stmt);
            }
        }

    return $affected_rows;
    }


//fetch row by auto binding parameters to provided mysql query and return numeric array with associative sub-arrays (where key is column name) with column values
function fetchRow($formatted_query, $column_values_array=null, $column_parameters_array=null)
    {
    $root_array=array();

    if (!empty($formatted_query))
        {
        $stmt=mysqli_prepare($GLOBALS["mysqli"], $formatted_query);
        if ($stmt)
            {
            if (!empty($column_values_array) && is_array($column_values_array) && !empty($column_parameters_array) && is_array($column_parameters_array) && count($column_values_array)==count($column_parameters_array)) //fetch specified row by binding parameters
                {
                call_user_func_array("mysqli_stmt_bind_param", array_merge(array($stmt, implode($column_parameters_array)), valuesToReferences($column_values_array)));
                }
            $exec=mysqli_stmt_execute($stmt);
            $results=mysqli_stmt_get_result($stmt);
            $results_total=mysqli_num_rows($results);
            mysqli_stmt_close($stmt);
            }

        while ($row=mysqli_fetch_assoc($results))
            {
            foreach ($row as $key=>$value)
                {
                $item_array[$key]=$value;
                }

            $root_array[]=$item_array;
            }
        }

    return $root_array;
    }


//duplicate row by using pre-formatted INSERT query and replacing some columns with other values if needed. return number of duplicated rows
function duplicateRow($formatted_query, $column_values_array, $column_parameters_array, $replaced_column_names_array=null, $replaced_column_values_array=null)
    {
    $affected_rows=0;

    if (!empty($formatted_query))
        {
        preg_match_all("/\(([A-Za-z0-9_, ]+)\)/", $formatted_query, $matches, PREG_SET_ORDER); //get all column names from $formatted_query
        if (!empty($matches[0][1])) //remove columns that don't need to be cloned because the number of column names in $formatted_query and number of values in $column_values_array must be the same
            {
            $column_names_array=explode(",", $matches[0][1]);
            if (!empty($column_names_array))
                {
                $column_names_array=array_map("trim", $column_names_array);

                foreach ($column_values_array as $key=>$value)
                    {
                    if (!in_array($key, $column_names_array))
                        {
                        unset($column_values_array[$key]);
                        }
                    }
                }
            }

        if (!empty($replaced_column_names_array) && is_array($replaced_column_names_array) && !empty($replaced_column_values_array) && is_array($replaced_column_values_array) && count($replaced_column_names_array)==count($replaced_column_values_array))
            {
            foreach ($replaced_column_names_array as $key=>$value) //replace some column values
                {
                if (array_key_exists($value, $column_values_array) && array_key_exists($key, $replaced_column_values_array)) //original column exists and replacement value for this column also exists
                    {
                    $column_values_array[$value]=$replaced_column_values_array[$key];
                    }
                }
            }

        $affected_rows+=insertRow($formatted_query, $column_values_array, $column_parameters_array);
        }

    return $affected_rows;
    }


//delete files and directories from specified directory ($files_array is an array of files and/or sub-directories to be deleted from $root_directory)
function deleteFileDirectory($root_directory, $files_array)
    {
    $removed_records=0;

    if (!empty($files_array))
        {
        $files_array=array_filter($files_array); //remove empty files (if any) from $files_array to prevent parent directory from being deleted too
        $files_array=array_diff($files_array, array(".", "..", "")); //remove dot files (if any) from $files_array to prevent parent directory from being deleted too when $files_array contains "."
        $files_array=array_values($files_array); //re-index array to prevent errors of undefined array indices
        }

    if (!empty($root_directory) && is_dir($root_directory) && !empty($files_array)) //proceed deleting files/directories
        {
        foreach ($files_array as $file)
            {
            if (is_file("$root_directory/$file") && unlink("$root_directory/$file")) //this is a file, delete
                {
                $removed_records++;
                }

            if (is_dir("$root_directory/$file")) //this is a directory, enter it and delete all files inside first
                {
                foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator("$root_directory/$file", FilesystemIterator::SKIP_DOTS), RecursiveIteratorIterator::CHILD_FIRST) as $path)
                    {
                    $path->isDir() && !$path->isLink() ? rmdir($path->getPathname()) : unlink($path->getPathname());
                    }

                if (rmdir("$root_directory/$file"))
                    {
                    $removed_records++;
                    }
                }
            }
        }

    return $removed_records;
    }


//parse NUMBERS, LETTERS, ALPHABET (numbers + letters), SEPARATORS (number + letters + _ and - symbols) or ANYTHING from string ($item_number_to_return starts at 0!)
function parseStringValue($text, $parsing_rule, $item_number_to_return=0)
    {
    $parsed_value=null;
    $rules_array_bad=array("NUMBERS", "LETTERS", "ALPHABET", "SEPARATORS", "ANYTHING");
    $rules_array_good=array("([0-9]+)", "([A-Za-z]+)", "([A-Za-z0-9]+)", "([A-Za-z0-9_\-]+)", "(.*?)");

    if (!empty($text) && in_array($parsing_rule, $rules_array_bad))
        {
        $formatted_parsing_pattern=str_replace($rules_array_bad, $rules_array_good, preg_quote($parsing_rule, "/")); //format valid parsing pattern

        $total_matches=preg_match_all("/$formatted_parsing_pattern/i", $text, $matches_array, PREG_SET_ORDER);
        if ($total_matches>0)
            {
            if ($item_number_to_return>=$total_matches) //less matches found, return value of the last match
                {
                $item_number_to_return=$total_matches-1; //array from preg_match_all starts with 0
                }
            $parsed_value=trim($matches_array[$item_number_to_return][1]);
            }
        }

    return $parsed_value;
    }


//set variable value only if it has no value yet
function setValue($old_value, $new_value)
    {
    $value_to_return=null;

    if (empty($old_value))
        {
        $value_to_return=$new_value;
        }
    else
        {
        $value_to_return=$old_value;
        }

    return $value_to_return;
    }


//filter $_POST requests submitted via form and unset submitted data if verification fails. only use on pages where data is submitted using <form> element, and not directly (like API, callback, etc.)
function filterFormPostRequest($post_data, $ROOT_URL, $directory_name, $refer)
    {
    if (!filter_var($ROOT_URL, FILTER_VALIDATE_URL) || empty($directory_name) || !filter_var($refer, FILTER_VALIDATE_URL) || stristr($refer, "$ROOT_URL/$directory_name")===false) //refer doesn't match expected value, unset submitted data
        {
        $post_data=null;
        }

    return $post_data;
    }
