<?php
//custom plugins used by this app only


//create license report
function createLicenseReport($SMART_REPORTS, $product_id, $account_id, $license_code, $report_text, $report_failure)
    {
    $report_date=date("Y-m-d");
    $report_system=0; //license reports should never be system
    $rows_array=array(); //will be filled with existing reports if SMART_REPORTS enabled

    if ($SMART_REPORTS==1) //check if such report already exists today
        {
        $rows_array=fetchRow("SELECT * FROM apl_reports WHERE product_id=? AND (account_id=? OR license_code=?) AND report_date=? AND report_text=?", array($product_id, $account_id, $license_code, $report_date, $report_text), array("i", "i", "s", "s", "s"));
        }

    if (empty($rows_array)) //no excessive reports found (or SMART_REPORTS disabled)
        {
        insertRow("INSERT IGNORE INTO apl_reports (product_id, account_id, license_code, report_date, report_text, report_system, report_failure) VALUES (?, ?, ?, ?, ?, ?, ?)", array($product_id, $account_id, $license_code, $report_date, $report_text, $report_system, $report_failure), array("i", "i", "s", "s", "s", "i", "i"));
        }
    }


//create license callback
function createLicenseCallback($SMART_REPORTS, $product_id, $client_id, $license_code, $callback_ip, $callback_domain, $callback_status)
    {
    $callback_date=date("Y-m-d");
    $rows_array=array(); //will be filled with existing callbacks if SMART_REPORTS enabled

    if ($SMART_REPORTS==1) //check if such callback already exists today
        {
        $rows_array=fetchRow("SELECT * FROM apl_callbacks WHERE product_id=? AND (client_id=? OR license_code=?) AND callback_ip=? AND callback_domain=? AND callback_date=? AND callback_status=?", array($product_id, $client_id, $license_code, $callback_ip, $callback_domain, $callback_date, $callback_status), array("i", "i", "s", "s", "s", "s", "i"));
        }

    if (empty($rows_array)) //no excessive callbacks found (or SMART_REPORTS disabled)
        {
        insertRow("INSERT IGNORE INTO apl_callbacks (product_id, client_id, license_code, callback_ip, callback_domain, callback_date, callback_status) VALUES (?, ?, ?, ?, ?, ?, ?)", array($product_id, $client_id, $license_code, $callback_ip, $callback_domain, $callback_date, $callback_status), array("i", "i", "s", "s", "s", "s", "i"));
        }
    }


//generate signature to be sent to user's script
function generateServerSignature($product_id, $root_url, $client_email, $license_code)
    {
    global $ROOT_URL;
    $license_signature=null;
    $root_ips_array=gethostbynamel(getRawDomain($ROOT_URL));

    if (!empty($root_ips_array)) //IP(s) resolved successfully
        {
        $license_signature=hash("sha256", implode("", $root_ips_array).$product_id.$license_code.$client_email.$root_url.gmdate("Y-m-d"));
        }

    return $license_signature;
    }


//verify signature received from user's script
function verifyScriptSignature($license_signature, $product_id, $root_url, $client_email, $license_code)
    {
    global $ROOT_URL;
    $result=false;
    $root_ips_array=gethostbynamel(getRawDomain($ROOT_URL));

    if (!empty($root_ips_array) && !empty($license_signature))
        {
        if (hash("sha256", gmdate("Y-m-d").$root_url.$client_email.$license_code.$product_id.implode("", $root_ips_array))==$license_signature)
            {
            $result=true;
            }
        }

    return $result;
    }


//return license notification with formatted variables (also return server signature if everything OK)
function returnLicenseNotification($notification_case, $root_url, $ip_address, $client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit)
    {
    $rows_array=fetchRow("SELECT * FROM apl_notifications WHERE notification_id=?", array(1), array("i"));
    foreach ($rows_array as $row)
        {
        extract($row);
        }

    $bad_text_array=array("%ROOT_URL%", "%IP_ADDRESS%", "%CLIENT_EMAIL%", "%CLIENT_FNAME%", "%CLIENT_LNAME%", "%LICENSE_CODE%", "%PRODUCT_ID%", "%PRODUCT_TITLE%", "%PRODUCT_DESCRIPTION%", "%PRODUCT_URL_HOMEPAGE%", "%PRODUCT_URL_DOWNLOAD%", "%PRODUCT_VERSION%", "%LICENSE_EXPIRE_DATE%", "%LICENSE_CANCEL_DATE%", "%LICENSE_UPDATES_DATE%", "%LICENSE_SUPPORT_DATE%", "%LICENSE_LIMIT%");
    $good_text_array=array($root_url, $ip_address, $client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);

    $notification_text=str_ireplace($bad_text_array, $good_text_array, "<$notification_case>".$$notification_case."</$notification_case>"); //wrap message into tags like <notification_...>

    if ($notification_case=="notification_license_ok") //everything OK, also return license signature
        {
        $notification_text.="<license_signature>".generateServerSignature($product_id, $root_url, $client_email, $license_code)."</license_signature>";
        }

    return $notification_text;
    }


//format client
function formatClient($client_id, $client_fname, $client_lname, $client_email, $license_code)
    {
    if (filter_var($client_id, FILTER_VALIDATE_INT))
        {
        $client_formatted="$client_fname $client_lname ($client_email)";
        }
    else
        {
        $client_formatted=$license_code;
        }

    return $client_formatted;
    }


//send reminder emails to clients
function sendReminderEmails($EMAIL_FROM_NAME, $EMAIL_FROM_ADDRESS, $EMAIL_EXPIRING_LICENSE_DAYS, $EMAIL_EXPIRING_UPDATES_DAYS, $EMAIL_EXPIRING_SUPPORT_DAYS, $EXPIRATION_CHECK_DATE)
    {
    $date_today=date("Y-m-d");
    $expiring_licenses_array=array();
    $expiring_updates_array=array();
    $expiring_support_array=array();

    if ($EXPIRATION_CHECK_DATE<$date_today) //emails need to be sent
        {
        if (filter_var($EMAIL_EXPIRING_LICENSE_DAYS, FILTER_VALIDATE_INT)) //get expiring licenses
            {
            $max_date_allowed=date("Y-m-d", strtotime("+$EMAIL_EXPIRING_LICENSE_DAYS days"));

            $expiring_licenses_array=fetchRow("SELECT * FROM apl_licenses JOIN apl_products ON apl_licenses.product_id=apl_products.product_id JOIN apl_clients ON apl_licenses.client_id=apl_clients.client_id WHERE apl_licenses.client_id IS NOT NULL AND apl_licenses.license_code IS NULL AND license_expire_date BETWEEN ? AND ? AND license_expire_email_date=? AND license_status=?", array($date_today, $max_date_allowed, "0000-00-00", 1), array("s", "s", "s", "i"));
            }

        if (filter_var($EMAIL_EXPIRING_UPDATES_DAYS, FILTER_VALIDATE_INT)) //get expiring updates
            {
            $max_date_allowed=date("Y-m-d", strtotime("+$EMAIL_EXPIRING_UPDATES_DAYS days"));

            $expiring_updates_array=fetchRow("SELECT * FROM apl_licenses JOIN apl_products ON apl_licenses.product_id=apl_products.product_id JOIN apl_clients ON apl_licenses.client_id=apl_clients.client_id WHERE apl_licenses.client_id IS NOT NULL AND apl_licenses.license_code IS NULL AND license_updates_date BETWEEN ? AND ? AND license_updates_email_date=? AND license_status=?", array($date_today, $max_date_allowed, "0000-00-00", 1), array("s", "s", "s", "i"));
            }

        if (filter_var($EMAIL_EXPIRING_SUPPORT_DAYS, FILTER_VALIDATE_INT)) //get expiring support
            {
            $max_date_allowed=date("Y-m-d", strtotime("+$EMAIL_EXPIRING_SUPPORT_DAYS days"));

            $expiring_support_array=fetchRow("SELECT * FROM apl_licenses JOIN apl_products ON apl_licenses.product_id=apl_products.product_id JOIN apl_clients ON apl_licenses.client_id=apl_clients.client_id WHERE apl_licenses.client_id IS NOT NULL AND apl_licenses.license_code IS NULL AND license_support_date BETWEEN ? AND ? AND license_support_email_date=? AND license_status=?", array($date_today, $max_date_allowed, "0000-00-00", 1), array("s", "s", "s", "i"));
            }

        if (!empty($expiring_licenses_array) || !empty($expiring_updates_array) || !empty($expiring_support_array)) //at least one email needs to be sent
            {
            if (!empty($EMAIL_FROM_NAME) && filter_var($EMAIL_FROM_ADDRESS, FILTER_VALIDATE_EMAIL)) //basic variables ok
                {
                $rows_array=fetchRow("SELECT * FROM apl_emails WHERE email_id=?", array(1), array("i")); //get subjects and texts for reminder emails
                foreach ($rows_array as $row)
                    {
                    extract($row);
                    }

                $bad_text_array=array("%CLIENT_EMAIL%", "%CLIENT_FNAME%", "%CLIENT_LNAME%", "%LICENSE_CODE%", "%PRODUCT_ID%", "%PRODUCT_TITLE%", "%PRODUCT_DESCRIPTION%", "%PRODUCT_URL_HOMEPAGE%", "%PRODUCT_URL_DOWNLOAD%", "%PRODUCT_VERSION%", "%LICENSE_EXPIRE_DATE%", "%LICENSE_CANCEL_DATE%", "%LICENSE_UPDATES_DATE%", "%LICENSE_SUPPORT_DATE%", "%LICENSE_LIMIT%"); //variables used in reminder emails

                $transport=Swift_MailTransport::newInstance(); //create transport
                $mailer=Swift_Mailer::newInstance($transport); //create mailer using created transport

                foreach ($expiring_licenses_array as $row) //send expiring license emails
                    {
                    extract($row);

                    if (!empty($client_fname) && !empty($client_lname) && filter_var($client_email, FILTER_VALIDATE_EMAIL))
                        {
                        $good_text_array=array($client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);

                        $email_subject=str_ireplace($bad_text_array, $good_text_array, $email_expiring_license_subject);
                        $email_text=str_ireplace($bad_text_array, $good_text_array, $email_expiring_license_text);

                        $message=Swift_Message::newInstance($email_subject) //create message
                        ->setBody($email_text, "text/html");

                        $message->setFrom(array($EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME));
                        $message->setTo(array($client_email=>"$client_fname $client_lname"));

                        $mailer_result=$mailer->send($message); //send message
                        if (filter_var($mailer_result, FILTER_VALIDATE_INT))
                            {
                            createReport("Expiring $product_title license reminder sent to $client_fname $client_lname ($client_email).", 0, 1, 0);
                            }
                        else
                            {
                            createReport("Expiring $product_title license reminder could not be sent to $client_fname $client_lname ($client_email) because of mail() function failure on server.", 0, 1, 1);
                            }
                        //echo "From: $EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME<br>To:$client_email=>$client_fname $client_lname<br>$email_subject<br>$email_text<br><br><br>";
                        updateRow("UPDATE apl_licenses SET license_expire_email_date=? WHERE license_id=?", array($date_today, $license_id), array("s", "i"));

                        }
                    }

                foreach ($expiring_updates_array as $row) //send expiring updates emails
                    {
                    extract($row);

                    if (!empty($client_fname) && !empty($client_lname) && filter_var($client_email, FILTER_VALIDATE_EMAIL))
                        {
                        $good_text_array=array($client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);

                        $email_subject=str_ireplace($bad_text_array, $good_text_array, $email_expiring_updates_subject);
                        $email_text=str_ireplace($bad_text_array, $good_text_array, $email_expiring_updates_text);

                        $message=Swift_Message::newInstance($email_subject) //create message
                        ->setBody($email_text, "text/html");

                        $message->setFrom(array($EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME));
                        $message->setTo(array($client_email=>"$client_fname $client_lname"));

                        $mailer_result=$mailer->send($message); //send message
                        if (filter_var($mailer_result, FILTER_VALIDATE_INT))
                            {
                            createReport("Expiring $product_title updates reminder sent to $client_fname $client_lname ($client_email).", 0, 1, 0);
                            }
                        else
                            {
                            createReport("Expiring $product_title updates reminder could not be sent to $client_fname $client_lname ($client_email) because of mail() function failure on server.", 0, 1, 1);
                            }
                        //echo "From: $EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME<br>To:$client_email=>$client_fname $client_lname<br>$email_subject<br>$email_text<br><br><br>";
                        updateRow("UPDATE apl_licenses SET license_updates_email_date=? WHERE license_id=?", array($date_today, $license_id), array("s", "i"));

                        }
                    }

                foreach ($expiring_support_array as $row) //send expiring support emails
                    {
                    extract($row);

                    if (!empty($client_fname) && !empty($client_lname) && filter_var($client_email, FILTER_VALIDATE_EMAIL))
                        {
                        $good_text_array=array($client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);

                        $email_subject=str_ireplace($bad_text_array, $good_text_array, $email_expiring_support_subject);
                        $email_text=str_ireplace($bad_text_array, $good_text_array, $email_expiring_support_text);

                        $message=Swift_Message::newInstance($email_subject) //create message
                        ->setBody($email_text, "text/html");

                        $message->setFrom(array($EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME));
                        $message->setTo(array($client_email=>"$client_fname $client_lname"));

                        $mailer_result=$mailer->send($message); //send message
                        if (filter_var($mailer_result, FILTER_VALIDATE_INT))
                            {
                            createReport("Expiring $product_title support reminder sent to $client_fname $client_lname ($client_email).", 0, 1, 0);
                            }
                        else
                            {
                            createReport("Expiring $product_title support reminder could not be sent to $client_fname $client_lname ($client_email) because of mail() function failure on server.", 0, 1, 1);
                            }
                        //echo "From: $EMAIL_FROM_ADDRESS=>$EMAIL_FROM_NAME<br>To:$client_email=>$client_fname $client_lname<br>$email_subject<br>$email_text<br><br><br>";
                        updateRow("UPDATE apl_licenses SET license_support_email_date=? WHERE license_id=?", array($date_today, $license_id), array("s", "i"));
                        }
                    }
                }
            }

        updateRow("UPDATE apl_settings SET EXPIRATION_CHECK_DATE=?", array($date_today), array("s"));
        }
    }
